from collections import OrderedDict


class TypeBase:
    _typed_cls_suffix = None
    _typed_name = None

    def __init__(self, manager):
        self.manager = manager

    @classmethod
    def get_typed_name(cls):
        if cls._typed_name:
            return cls._typed_name
        return cls.__name__[:-len(cls._typed_cls_suffix)]

    @classmethod
    def on_typed_registered(cls, types):
        pass


class TypesBase:
    def __init__(self):
        self._maps = OrderedDict()

    def __getitem__(self, item):
        return self._maps[item]

    def keys(self):
        return self._maps.keys()

    def items(self):
        return self._maps.items()

    def get(self, typed):
        return self._maps.get(typed)

    def register(self, cls):
        self._maps[cls.get_typed_name()] = cls
        cls.on_typed_registered(self)
        return cls


class ManagerBase:
    types: TypesBase = None
